unit Use_Frustum;

interface

{
  0.01
  procedures find view frustum collision}

Uses
  OpenGL,
  Use_Geometry;


var
  Frustum: Array[0..5] of TPlane;
  //6 sides of view frustum
procedure Normalize_Plane(var p:TPlane);
procedure Find_Frustum;
function Point_In_Frustum(pt:TPoint): Boolean;overload;
function Point_In_Frustum(x,y,z:Single): Boolean;overload;
function Sphere_in_Frustum(pt:TPoint;Radius:Single): Boolean;overload;
function Sphere_in_Frustum(x,y,z,Radius:Single): Boolean;overload;

implementation

{$REGION 'plane normilize'}
procedure Normalize_Plane(var p:TPlane);
var
  s: Single;
begin
  s := Sqrt(Sqr(P[0]) + Sqr(P[1]) + Sqr(P[2]));
  P[0] := P[0] / s;
  P[1] := P[1] / s;
  P[2] := P[2] / s;
  P[3] := P[3] / s;
end;
{$ENDREGION}
{$REGION 'Find_Frustum'}
procedure Find_Frustum;
var
  p,m,c: Array[0..15] Of Single;
begin
  glGetFloatv(GL_PROJECTION_MATRIX, @p);
  glGetFloatv(GL_MODELVIEW_MATRIX, @m);

  c[0] := m[0] * p[0] + m[1] * p[4] + m[2] * p[8] + m[3] * p[12];
  c[1] := m[0] * p[1] + m[1] * p[5] + m[2] * p[9] + m[3] * p[13];
  c[2] := m[0] * p[2] + m[1] * p[6] + m[2] * p[10] + m[3] * p[14];
  c[3] := m[0] * p[3] + m[1] * p[7] + m[2] * p[11] + m[3] * p[15];
  c[4] := m[4] * p[0] + m[5] * p[4] + m[6] * p[8] + m[7] * p[12];
  c[5] := m[4] * p[1] + m[5] * p[5] + m[6] * p[9] + m[7] * p[13];
  c[6] := m[4] * p[2] + m[5] * p[6] + m[6] * p[10] + m[7] * p[14];
  c[7] := m[4] * p[3] + m[5] * p[7] + m[6] * p[11] + m[7] * p[15];
  c[8] := m[8] * p[0] + m[9] * p[4] + m[10] * p[8] + m[11] * p[12];
  c[9] := m[8] * p[1] + m[9] * p[5] + m[10] * p[9] + m[11] * p[13];
  c[10] := m[8] * p[2] + m[9] * p[6] + m[10] * p[10] + m[11] * p[14];
  c[11] := m[8] * p[3] + m[9] * p[7] + m[10] * p[11] + m[11] * p[15];
  c[12] := m[12] * p[0] + m[13] * p[4] + m[14] * p[8] + m[15] * p[12];
  c[13] := m[12] * p[1] + m[13] * p[5] + m[14] * p[9] + m[15] * p[13];
  c[14] := m[12] * p[2] + m[13] * p[6] + m[14] * p[10] + m[15] * p[14];
  c[15] := m[12] * p[3] + m[13] * p[7] + m[14] * p[11] + m[15] * p[15];

  Frustum[0][0] := c[3] - c[0];
  Frustum[0][1] := c[7] - c[4];
  Frustum[0][2] := c[11] - c[8];
  Frustum[0][3] := c[15] - c[12];
  Normalize_Plane(Frustum[0]);

  Frustum[1][0] := c[3] + c[0];
  Frustum[1][1] := c[7] + c[4];
  Frustum[1][2] := c[11] + c[8];
  Frustum[1][3] := c[15] + c[12];
  Normalize_Plane(Frustum[1]);

  Frustum[2][0] := c[3] + c[1];
  Frustum[2][1] := c[7] + c[5];
  Frustum[2][2] := c[11] + c[9];
  Frustum[2][3] := c[15] + c[13];
  Normalize_Plane(Frustum[2]);

  Frustum[3][0] := c[3] - c[1];
  Frustum[3][1] := c[7] - c[5];
  Frustum[3][2] := c[11] - c[9];
  Frustum[3][3] := c[15] - c[13];
  Normalize_Plane(Frustum[3]);

  Frustum[4][0] := c[3] - c[2];
  Frustum[4][1] := c[7] - c[6];
  Frustum[4][2] := c[11] - c[10];
  Frustum[4][3] := c[15] - c[14];
  Normalize_Plane(Frustum[4]);

  Frustum[5][0] := c[3] + c[2];
  Frustum[5][1] := c[7] + c[6];
  Frustum[5][2] := c[11] + c[10];
  Frustum[5][3] := c[15] + c[14];
  Normalize_Plane(Frustum[5]);
end;
{$ENDREGION}
{$REGION 'Point_in_Frustum'}
function Point_In_Frustum(pt:TPoint): Boolean;overload;
var
  i: Integer;
begin
  Result := True;     
  for i := 0 to 5 do
    if (Frustum[i][0] * pt[0] + Frustum[i][1] * pt[1] + Frustum[i][2] * pt[2] + Frustum[i][3]) <= 0 then
    begin
      Result := False;
      Exit;
    end;
end;

function Point_In_Frustum(x,y,z:Single): Boolean;overload;
var
  i: Integer;
begin
  Result := True;     
  for i := 0 to 5 do
    if (Frustum[i][0] * x + Frustum[i][1] * y + Frustum[i][2] * z + Frustum[i][3]) <= 0 then
    begin
      Result := False;
      Exit;
    end;
end;
{$ENDREGION}
{$REGION 'Sphere_in_Frustum'}
function Sphere_in_Frustum(pt:TPoint;Radius:Single): Boolean;overload;
var
  i: Integer;
begin
  Result := True;     
  for i := 0 to 5 do
    if (Frustum[i][0] * pt[0] + Frustum[i][1] * pt[1] + Frustum[i][2] * pt[2] + Frustum[i][3]) <= -Radius then
    begin
      Result := False;
      Exit;
    end;
end;

function Sphere_in_Frustum(x,y,z,Radius:Single): Boolean;overload;
var
  i: Integer;
begin
  Result := True;     
  for i := 0 to 5 do
    if (Frustum[i][0] * x + Frustum[i][1] * y + Frustum[i][2] * z + Frustum[i][3]) <= -Radius then
    begin
      Result := False;
      Exit;
    end;
end;
{$ENDREGION}
end.
